pub struct CircularBuffer<T> { /* private fields */ }Implementations
Sourceimpl<T> CircularBuffer<T>
impl<T> CircularBuffer<T>
Sourcepub fn new(size: usize) -> Self
pub fn new(size: usize) -> Self
Create a new CircularBuffer of size size.
It allocate an array of exactly size element, if the allocation fail, the method panic.
Negligible amount of space used by the CircularBuffer beside the array itself.
Sourcepub fn push(&mut self, value: T) -> usize
pub fn push(&mut self, value: T) -> usize
Push a new element into the CircularBuffer in O(1) does not do any allocation.
If the CircularBuffer is full, the first element of the CircularBuffer is overwritten.
Sourcepub fn fill(&mut self, return_vector: &mut Vec<T>) -> usize
pub fn fill(&mut self, return_vector: &mut Vec<T>) -> usize
Main method to read elements out of the CircularBuffer.
The return vector get filled, with as many as possible elements from the CircularBuffer.
The available elements in the CircularBuffer are the same returned by the method len. The elements
that the vector can accepts are given by return_vector.capacity() - return_vector.len()
The method avoids allocating memory.
Hence if the vector is already full, no elements are pushed into the vector.
If the CircularBuffer is empty, no elements are pushed into the vector.
If the vector can accept more elements that are prensent in the CircularBuffer, the vector get filled as much as possible and the CircularBuffer will remain empty.
If the vector cannot accept all the element in the CircularBuffer, the vector get filled while the CircularBuffer will be left with some elements inside.
The operation runs in O(n) with n number of elements pushed into the vector.
Sourcepub fn _fast_fill(&mut self, return_vector: &mut Vec<T>) -> usize
pub fn _fast_fill(&mut self, return_vector: &mut Vec<T>) -> usize
The _fast_fill method is supposed to be a faster alternative to the fill one.
However, benchmarks failed to show any difference in performance.
If the benchmark showed any difference, it was the _fast_fill method being a little slower.
The _fast_fill method is more complex that the fill method, so I suggest to rely on the
simpler fill. However both methods passed the same properties tests, so they should be
equally correct.
The _fast_fill is implemented using raw pointer and memcopy. While the fill method
pull elements using the iterator and simply push them to the back of the vector.